<?php

namespace Drupal\entity_holder\Controller;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityFormBuilderInterface;
use Drupal\Core\Render\RenderContext;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\Core\Url;
use Drupal\entity_holder\EntityHolderInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\HttpKernel\HttpKernelInterface;

/**
 * Returns responses for Entity Holder routes.
 */
class EntityHolderController extends ControllerBase {

  /**
   * Symfony\Component\HttpKernel\HttpKernelInterface definition.
   *
   * @var \Symfony\Component\HttpKernel\HttpKernelInterface
   */
  protected $httpKernel;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The entity form builder.
   *
   * @var \Drupal\Core\Entity\EntityFormBuilderInterface
   */
  protected $entityFormBuilder;

  /**
   * EntityHolderController constructor.
   *
   * @param \Drupal\Core\Entity\EntityFormBuilderInterface $entity_form_builder
   *   The entity form builder.
   */
  public function __construct(HttpKernelInterface $http_kernel, RendererInterface $renderer, EntityFormBuilderInterface $entity_form_builder) {
    $this->httpKernel = $http_kernel;
    $this->renderer = $renderer;
    $this->entityFormBuilder = $entity_form_builder;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('http_kernel.basic'),
      $container->get('renderer'),
      $container->get('entity.form_builder')
    );
  }

  /**
   * Builds the response.
   */
  public function buildHeldEntityForm(EntityHolderInterface $entity_holder) {
    if ($held_entity = $entity_holder->getHeldEntity()) {
      // If the held entity already exists, redirect to its edit form.
      $url = $held_entity->toUrl('edit-form')->toString(TRUE);
      return new TrustedRedirectResponse($url->getGeneratedUrl());
    }

    // Build the held entity create form.
    $held_entity_type_storage = $this->entityTypeManager()
      ->getStorage($entity_holder->getHeldEntityTypeId());
    $held_entity_type = $held_entity_type_storage->getEntityType();
    $held_entity = $held_entity_type_storage->create([
      'uuid' => $entity_holder->getHeldEntityUuid(),
      $held_entity_type->getKey('bundle') => $entity_holder->getHeldEntityBundle(),
      // Set the holder title as default title for the the held entity.
      $held_entity_type->getKey('label') => $entity_holder->getTitle(),
    ]);

    $form = $this->entityFormBuilder->getForm($held_entity);
    $form['#title'] = $this->t('Create @name', ['@name' => $held_entity_type->getLabel()]);

    return $form;
  }

  /**
   * Entity holder route title callback.
   *
   * @param \Drupal\entity_holder\EntityHolderInterface $entity_holder
   *   The entity holder.
   *
   * @return array
   *   The held entity title as render array, the entity holder title if there
   *   is no held entity.
   */
  public function entityHolderTitle(EntityHolderInterface $entity_holder) {
    $held_entity = $entity_holder->getHeldEntity();

    return [
      '#markup' => $held_entity ? $held_entity->label() : $entity_holder->label(),
      '#allowed_tags' => Xss::getHtmlTagList()
    ];
  }

  /**
   *
   * @param \Drupal\entity_holder\EntityHolderInterface $entity_holder
   *
   * @return array|mixed
   */
  public function entityHolderView(EntityHolderInterface $entity_holder) {
    if ($held_entity = $entity_holder->getHeldEntity()) {
      // Compose response from subrequest to the held entity canonical URL.
      $context = new RenderContext();
      $response = $this->renderer->executeInRenderContext($context, function () use ($held_entity) {
        $sub_request = Request::create($held_entity->toUrl()->setAbsolute()->toString());
        return $this->httpKernel->handle($sub_request, HttpKernelInterface::SUB_REQUEST);
      });

      if (!$context->isEmpty()) {
        $metadata = $context->pop();
        // Add entity holder cache information.
        $metadata->addCacheableDependency($entity_holder);
        $response->addCacheableDependency($metadata);
        $response->addAttachments($metadata->getAttachments());
      }

      return $response;
    }

    // Response with the fallback content if set.
    $build = $entity_holder->getFallbackContent() ?: [];

    // Set message the user.
    $create_url = Url::fromRoute('entity.entity_holder.held_entity_form', ['entity_holder' => $entity_holder->id()]);
    if ($create_url->access()) {
      $this->messenger()->addWarning($this->t('The content associated to this entity holder does not exist yet. <a href=":url">Create this content</a>.', [':url' => $create_url->toString()]));
    }
    elseif (empty($build)) {
      // There is no fallback content and the user cannot create the hosted
      // entity, return page not found.
      throw new NotFoundHttpException();
    }

    return $build;
  }

}
